########################################################################
#
# File Name:            Backup.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""


__doc__ = """Serialize every object in the system into an XML format.  This will dump the serialized objects to a directory."""

import os,string
from Ft.Server import RESERVED_NAMESPACE
from Ft.Server.Server import FtServerServerException
from Ft.Server.Console import CommandUtil, Authenticate
from Ft.Server.Core import Constants
from distutils import dir_util
from Ft.Xml import Domlette
def Run(options,args):

    d = args['dir']
    if os.path.exists(d):
        if not options.has_key('force'):
            print "Directory %s already exists" % d
            resp = raw_input("Remove the directory? (yes/no)")
            if string.lower(resp) not in ['yes','y']:
                return
        dir_util.remove_tree(d)
    os.makedirs(d)

    #Do authentication our selve
    options['AUTHENTICATION'] = Authenticate.UserAuthenticate(options=options)

    try:
        repo = CommandUtil.GetRepository(options)
        try:
            _SerializeSysObject(repo,d)
            _SerializeRdf(repo,d)
            _SerializeContainer(repo,d)
        finally:
            repo.txRollback()
    except FtServerServerException, e:
        print e
        return

def _SerializeRdf(repo,directory):
    model = repo.getModel()
    print "Serializing RDF"
    dom = model.serialize(None)
    file = open(os.path.join(directory,'ftss-system.rdf'),'w')
    file.write(dom)
    file.close()

def _SerializeContainer(cont,d):

    print "Serializing %s" % cont.getUri()

    #First serialize the cont itself
    f = open(os.path.join(d,'_ftss_container.xml'),'w')
    cont.serialize(stream=f)
    f.close()

    for cr in cont:
        c = cont.fetchResource(cr.uri)
        if c.isResourceType(Constants.ResourceType.FILE) or c.isResourceType(Constants.ResourceType.ALIAS):
            print "Serializing %s" % c.getUri()
            fileName = os.path.join(d,os.path.split(c.getUri())[1])
            f = open(fileName,'w')
            c.serialize(stream=f)
            f.close()
        elif c.isResourceType(Constants.ResourceType.CONTAINER):
            newd = os.path.join(d,os.path.split(c.getUri())[1])
            os.mkdir(newd)
            _SerializeContainer(c,newd)
        


def _SerializeSysObject(repo,d):
    print "Serialize System Object"
    so = repo.getSystemObject()

    doc = Domlette.Document()
    de = doc.createElementNS(RESERVED_NAMESPACE,'ft:SystemObject')
    doc.appendChild(de)

    for aclKey in ['read',
                   'write',
                   'start',
                   'stop',
                   'init']:
        for a in so.getAcl(aclKey):
            acl = de.ownerDocument.createElementNS(RESERVED_NAMESPACE,'ft:Acl')
            acl.setAttributeNS('',aclKey,a)
            de.appendChild(acl)
            

    fileName = os.path.join(d,'ftss-system-object.xml')
    f = open(fileName,'w')
    ext.Print(doc,f)
    


def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('backup',
                          """Serialize the entire object in the system.""",
                          '/tmp/4SSBackup',
                          __doc__,
                          function=Run,
                          arguments = [Arguments.RequiredArgument('dir',
                                                                  "The directory to create and send all of the data to",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,'force','Force removing the directory if it exists.')]),
                          )
    return cmd
