"""
The stop command is used to stop any or all of the 4SS servers.  To stop
all the servers, execute the stop command with out any arguments.  To stop
an individual server, specify their names as command line arguments.  You
will be prompted for authentication before you can stop the servers.
"""

import os, sys

from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Core import Constants
from Ft.Server.Core.Lib import ConfigFile


def Run(serverNames=None, configFile=None, coreId=None,display=1):
    from Ft.Server.Protocols import StartMode, Launcher


    properties = ConfigFile.ReadProperties(fileName=configFile)
    coreId = coreId or Constants.DEFAULT_CORE_ID

    serverDefs = {}
    for defs in properties.values():
        serverDefs.update(defs)

    if not serverNames:
        for name, prop in serverDefs.items():
            if prop.get('StartMode') == StartMode.AUTOMATIC:
                StopServer(name,prop,configFile,coreId,display)
    else:
        for name in serverNames:
            prop = serverDefs.get(name)
            if not prop:
                raise FtServerServerException(Error.UNKNOWN_SERVER, server=name)
            StopServer(name,prop,configFile,coreId,display)
    return


def StopServer(name, properties,configFile,coreId,display):
    from Ft.Server.Protocols import Launcher
    launcher = Launcher.Launcher(name, properties, display,configFileName=configFile,coreId = coreId)
    (pid, status) = launcher.status()
    if status == Launcher.PID_RUNNING:
        launcher.stop()
    return


def TextStop(options, args):
    try:
        serverNames = args.get('serverName')
        configFile = options.get('config-file')
        coreId = options.get('core-id', 'Core')
        Run(serverNames, configFile, coreId)
    except FtServerServerException, e:
        print e
    return



def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('stop',
                          "Stop protocol server(s)",
                          'ApacheServer',
                          __doc__,
                          function=TextStop,
                          arguments = [Arguments.ZeroOrMoreArgument('serverName',
                                                                    'Name of server to wuery',
                                                                    str),
                                       ],
                          authenticate='stop',
                          )
    return cmd

