from Ft.Server.Console.Widgets import *

from Ft.Server.Server import FtServerServerException, Error
from Ft.Server import __version__
from Ft.Server.Core import Constants
from Ft.Server.Core.Lib import ConfigFile, SystemInit
from Ft.Server.Protocols import Launcher, StartMode

import string, sha, os, stat

SERVICE_DESC = 0
SERVICE_STATUS = 1
SERVICE_MODE = 2

def StatusToString(status):
    if status == Launcher.PID_RUNNING:
        return 'Running'
    if status == Launcher.PID_STOPPED:
        return 'Stopped'
    return 'Unknown'

def StartupToString(mode):
    if mode == StartMode.AUTOMATIC:
        return 'Automatic'
    if mode == StartMode.MANUAL:
        return 'Manual'
    return 'Unknown'

class ManagementConsole(TApplication):
    name = "Management Console"
    def __init__(self):
        TApplication.__init__(self)
        # FIXME - we should probably specify this somehow
        self._config_fname = ConfigFile.GetFileName()
        self._config_mtime = -1
        self.getServerConfig()
        config = self.getConfig()

    def getConfig(self, mtime=0):
        mtime = mtime or os.stat(self._config_fname)[stat.ST_MTIME]
        if mtime > self._config_mtime:
            self._config_props = ConfigFile.ReadProperties(fileName=self._config_fname)
            self._config_mtime = os.stat(self._config_fname)[stat.ST_MTIME]
        return self._config_props

    def getServerConfig(self, name=None):
        mtime = os.stat(self._config_fname)[stat.ST_MTIME]
        if mtime > self._config_mtime:
            config = self.getConfig(mtime)
            merged = {}
            for server in config.values():
                merged.update(server)
            for server, props in merged.items():
                if not props.has_key('StartMode'):
                    del merged[server]
            self._config_merged = merged
        if name:
            return self._config_merged[name]
        return self._config_merged

    def getLauncher(self, name):
        config = self.getServerConfig(name)
        return Launcher.Launcher(name, config)

class TConsole(TForm):
    def __init__(self, owner):
        TForm.__init__(self, owner,
                       left=213,
                       top=400,
                       borderIcons=[biSystemMenu, biMinimize],
                       borderStyle=bsSizeable,
                       caption='4Suite Server Management Console',
                       clientHeight=226,
                       clientWidth=516,
                       color=clBtnFace,
                       position=poScreenCenter,
                       textHeight=13,
                       )
        self.onCreate = self.onConsoleCreate
        self.onClose = self.onConsoleClose

        self.largeIcons = None
        self.smallIcons = None

        # Popup menu for the services
        self.servicePopup = TPopupMenu(self)
        TMenuItem(self.servicePopup,
                  label='&Properties',
                  command=self.tbPropertiesClick)
        TMenuItem(self.servicePopup, label='-')
        self.popupStart = TMenuItem(self.servicePopup,
                                    label='&Start',
                                    command=self.tbStartClick)
        self.popupStop = TMenuItem(self.servicePopup,
                                   label='S&top',
                                   command=self.tbStopClick)
        self.popupRestart = TMenuItem(self.servicePopup,
                                      label='&Restart',
                                      command=self.tbRestartClick)

        self.services = TListView(self,
                                  left=0,
                                  top=26,
                                  width=516,
                                  height=181,
                                  align=alClient,
                                  columns=[TListColumn(caption='Service Name',
                                                       width=120),
                                           TListColumn(autoSize=1,
                                                       caption='Description'),
                                           TListColumn(alignment=taCenter,
                                                       caption='Status',
                                                       width=60),
                                           TListColumn(alignment=taCenter,
                                                       caption='Startup',
                                                       width=70),
                                           ],
                                  largeImages=self.largeIcons,
                                  rowSelect=1,
                                  popupMenu=self.servicePopup,
                                  onSelect = self.lvSelect,
                                  smallImages=self.smallIcons,
                                  tabOrder=0,
                                  scrollbar = ssVertical,
                                  viewStyle=vsReport)


        # Create the toolbar
        self.toolBarImages = None
        
        toolBar = TToolBar(self,
                           left=0,
                           top=0,
                           width=516,
                           height=26,
                           autoSize=1,
                           edgeBorders=[ebLeft, ebTop, ebRight, ebBottom],
                           images=self.toolBarImages,
                           showHint=1,
                           tabOrder=1)
        # Properties
        TToolButton(toolBar,
                    left=0,
                    top=0,
                    hint='Properties',
                    caption='Properties',
                    imageIndex=0,
                    onClick = self.tbPropertiesClick)
        # Separator 1
        TToolButton(toolBar,
                    left=23,
                    top=0,
                    width=8,
                    style=tbsSeparator)
        # Help
        TToolButton(toolBar,
                    left=31,
                    top=0,
                    hint='Help',
                    caption='Help',
                    imageIndex=1,
                    onClick = self.tbHelpClick)
        # Separator 2
        TToolButton(toolBar,
                    left=54,
                    top=0,
                    width=8,
                    style=tbsSeparator)
        # Start
        TToolButton(toolBar,
                    left=62,
                    top=0,
                    hint='Start Service',
                    caption='Start',
                    imageIndex=2,
                    onClick = self.tbStartClick)
        # Stop
        TToolButton(toolBar,
                    left=85,
                    top=0,
                    hint='Stop Service',
                    caption='Stop',
                    imageIndex=3,
                    onClick = self.tbStopClick)
        # Restart
        TToolButton(toolBar,
                    left=108,
                    top=0,
                    hint='Restart Service',
                    caption='Restart',
                    imageIndex=4,
                    onClick = self.tbRestartClick)

        # The status bar
        self.statusBar = TStatusBar(self,
                                    left=0,
                                    top=207,
                                    height=19,
                                    width=516,
                                    )

        # The main menu
        self.mainMenu = TMainMenu(self)
        self.fileMenu = TMenuItem(self.mainMenu, label='&File')
        TMenuItem(self.fileMenu, label='&Options...')
        TMenuItem(self.fileMenu, label='-')
        TMenuItem(self.fileMenu, label='E&xit', command=self.close)

        self.globalMenu = TMenuItem(self.mainMenu, label='&Global')
        self.globalMenuStart = TMenuItem(self.globalMenu,
                                         label='&Start',
                                         command=self.globalStart)
        self.globalMenuStop = TMenuItem(self.globalMenu,
                                        label='S&top',
                                        command=self.globalStop)
        self.globalMenuRestart = TMenuItem(self.globalMenu,
                                           label='&Restart',
                                           command=self.globalRestart)
        self.globalMenuInit = TMenuItem(self.globalMenu,
                                        label='&Initialize Database',
                                        command=self.globalInit)
        TMenuItem(self.globalMenu, label='-')
        self.globalMenuRefresh = TMenuItem(self.globalMenu,
                                           label='Re&fresh',
                                           command=self.globalRefresh)

        self.serviceMenu = TMenuItem(self.mainMenu, label='&Service')
        self.serviceMenuStart = TMenuItem(self.serviceMenu, label='&Start',command=self.tbStartClick)
        self.serviceMenuStop = TMenuItem(self.serviceMenu, label='S&top',command=self.tbStopClick)
        self.serviceMenuRestart = TMenuItem(self.serviceMenu, label='&Restart',command=self.tbRestartClick)
        TMenuItem(self.serviceMenu, label='-')
        self.serviceMenuProperties = TMenuItem(self.serviceMenu, label='&Properties')

        self.helpMenu = TMenuItem(self.mainMenu, label='&Help')
        TMenuItem(self.helpMenu, label='&Contents')
        TMenuItem(self.helpMenu, label='-')
        TMenuItem(self.helpMenu, label='&About...', command=self.aboutClick)


    def _updateAllServices(self, status=None):
        config = Application.getServerConfig()
        self.services.items.beginUpdate()
        self.services.selected = None
        l = len(self.services.items)
        for ctr in range(l):
            self.services.items.remove(0)

        names = config.keys()
        names.sort()
        for name in names:
            item = TListItem()
            item.caption = name
            item.subitems = ['']*len(self.services.columns)
            launcher = Launcher.Launcher(name, config[name])
            self._updateService(item, launcher)
            if status:
                item.subitems[SERVICE_STATUS] = status
            self.services.items.append(item)
        self.services.items.endUpdate()
        if status:
            self.services.redraw()
        self._updateMenusAndButtons()
        return

    def _updateService(self, item, launcher):
        name = item.caption
        config = Application.getServerConfig(name)
        item.subitems[SERVICE_DESC] = config['ServerType']
        item.subitems[SERVICE_STATUS] = StatusToString(launcher.status()[1])
        item.subitems[SERVICE_MODE] = StartupToString(config['StartMode'])
        return

    def tbPropertiesClick(self):
        print 'properties of',self.services.selected.caption
        return

    def tbHelpClick(self):
        return

    def tbStartClick(self):
        name = self.services.selected.caption
        self.services.selected.subitems[SERVICE_STATUS] = 'Starting'
        self.services.redraw()
        launcher = Application.getLauncher(name)
        try:
            launcher.start()
        except:
            pass
        self._updateService(self.services.selected, launcher)
        self.services.redraw()
        return

    def tbStopClick(self):
        name = self.services.selected.caption
        launcher = Application.getLauncher(name)
        if launcher.status()[1] == Launcher.PID_RUNNING:
            launcher.stop()
            self._updateService(self.services.selected, launcher)
            self.services.redraw()
        return

    def tbRestartClick(self):
        name = self.services.selected.caption
        self.services.selected.subitems[SERVICE_STATUS] = 'Starting'
        self.services.redraw()
        launcher = Application.getLauncher(name)
        launcher.restart()
        self._updateService(self.services.selected, launcher)
        self.services.redraw()
        return

    def globalStart(self):
        config = Application.getServerConfig()
        self.services.items.beginUpdate()
        items = []
        for item in self.services.items:
            name = item.caption
            launcher = Launcher.Launcher(name, config[name])
            if launcher.status()[1] != Launcher.PID_RUNNING:
                item.subitems[SERVICE_STATUS] = 'Starting'
                items.append((item, launcher))

        for (item, launcher) in items:
            self.services.redraw()
            try:
                launcher.start()
            except:
                pass
            item.subitems[SERVICE_STATUS] = StatusToString(launcher.status()[1])

        self.services.redraw()
        self._updateMenusAndButtons()
        return
        
        for name in Application.getServerConfig().keys():
            launcher = Application.getLauncher(name)
            launcher.start()
        self._updateAllServices()
        return

    def globalStop(self,count = 0):
        for name in Application.getServerConfig().keys():
            launcher = Application.getLauncher(name)
            if launcher.status()[1] == Launcher.PID_RUNNING:
                launcher.stop()
        self._updateAllServices()
        return

    def globalRestart(self):
        for name in Application.getServerConfig().keys():
            launcher = Application.getLauncher(name)
            launcher.restart()
        self._updateAllServices()
        return
    
    def globalInit(self):
        mr = Application.messageBox('Question',
                                    'This will erase all data in the system. Are you sure?',
                                    icon=mbQuestion,
                                    buttons=mbYesNo,
                                    default=mbNo)
        if mr != mrYes:
            return

        #FIXME we should check that a user server is defined
        if SysInit.showModal() == mrCancel:
            return
        
        self.redraw()
        core = Application.getConfig()['Core'][Constants.DEFAULT_CORE_ID]
        # A list of tuples [(username, pwd_hash), ...]
        users = map(None,
                    SysInit.superUsers.items.strings,
                    SysInit.superUsers.items.objects)
        SystemInit.Init(core['DbDriver'],
                        core['DatabaseName'],
                        core['SystemContainer'],
                        users)
        return

    def globalRefresh(self):
        return self._updateAllServices()

    def lvSelect(self,sender,item):
        self._updateMenusAndButtons()

    def aboutClick(self):
        return

    def onConsoleCreate(self, sender):
        self._updateAllServices()
        return

    def onConsoleClose(self, sender):
        return

    def _updateMenusAndButtons(self):
        if self.services.selected:
            launcher = Application.getLauncher(self.services.selected.caption)
            running = launcher.status()[1] == Launcher.PID_RUNNING
            selected = 1
            self.serviceMenuStart.enable(not running)
            self.serviceMenuStop.enable(running)
            self.serviceMenuRestart.enable(running)
            self.popupStart.enable(not running)
            self.popupStop.enable(running)
            self.popupRestart.enable(running)
        else:
            selected = 0

        self.serviceMenu.enable(selected)
        return

def Run():
    from Ft.Server.Console.Forms import Initialize

    global Application, Console, SysInit

    Application = ManagementConsole()

    Initialize.Application = Application

    # Now we know that Tk and a window manager are available
    print "Starting graphical console"

    Console = Application.createForm(TConsole)
    SysInit = Application.createForm(Initialize.TInitialize)

    Application.run()
