_rgb = lambda r, g, b : '#%02x%02x%02x' % (r,g,b)
clBlack = _rgb(0, 0, 0)
clMaroon = _rgb(128, 0, 0)
clGreen = _rgb(0, 128, 0)
clOlive = _rgb(128, 128, 0)
clNavy = _rgb(0, 0, 128)
clPurple = _rgb(128, 0, 128)
clTeal = _rgb(0, 128, 128)
clDkGray = clGray = _rgb(128, 128, 128)
clLtGray = clSilver = _rgb(192, 192, 192)
clRed = _rgb(255, 0, 0)
clLime = _rgb(0, 255, 0)
clYellow = _rgb(255, 255, 0)
clBlue = _rgb(0, 0, 255)
clFuchsia = _rgb(255, 0, 255)
clAqua = _rgb(0, 255, 255)
clWhite = _rgb(255, 255, 255)

# System dependent colors
import sys
if sys.platform == 'win32':
    clBtnFace = 'SystemButtonFace'
    clWindowText = 'SystemWindowText'
    clWindow = 'SystemWindow'
    _default_font = 'MS Sans Serif'
else:
    clBtnFace = clLtGray
    clWindowText = clBlack
    clWindow = clWhite
    _default_font = 'Helvetica'
del sys

def _enum(*args):
    num = 0
    for name in args:
        globals()[name] = num
        num = num + 1
    return

# TFontStyle enum { fsBold, fsItalic, fsUnderline, fsStrikeout };
_enum('fsBold',
      'fsItalic',
      'fsUnderline',
      'fsStrikeout',
      )

from tkFont import Font
_Font = Font
del Font

import Tkinter
_tk = Tkinter
del Tkinter

class TFont(_Font):
    def __init__(self, **options):
        root = _tk._default_root
        name = 'font' + str(id(self))
        self._name = name
        self._root  = root
        self._split = root.tk.splitlist
        self._call  = root.tk.call
        if options.has_key('color'):
            self.color = options['color']
            del options['color']
        else:
            self.color = clWindowText
        config = {'name' : _default_font,
                  'size' : 12,
                  'style' : [],
                  }
        config.update(options)
        self.__dict__.update(config)
        config['family'] = config['name']
        # name is internal to tk
        del config['name']
        apply(self._call, ('font', 'create', name) + self._set(config))

    def __str__(self):
        "Allows this to be used as a Tkinter font"
        return self._name

    def __del__(self):
        try:
            self._call('font', 'delete', self._name)
        except (AttributeError, _tk.TclError):
            pass

    def _set(self, config):
        options = []
        for key, value in config.items():
            if key == 'style':
                styles = {'-weight' : 'normal',
                         '-slant' : 'roman',
                         '-underline' : '0',
                         '-overstrike' : '0',
                         }
                if fsBold in value:
                    styles['-weight'] = 'bold'
                if fsItalic in value:
                    styles['-slant'] = 'italic'
                if fsUnderline in value:
                    styles['-underline'] = '1'
                if fsStrikeout in value:
                    styles['-overstrike'] = '1'
                for pair in styles.items():
                    options.extend(list(pair))
            elif key == 'name':
                options.extend(['-family', value])
            else:
                options.append('-' + key)
                options.append(str(value))
        return tuple(options)

    def __setattr__(self, name, value):
        if name in ['size', 'style', 'name']:
            apply(self._call, ('font', 'config', self._name) +
                  self._set({name : value}))
        self.__dict__[name] = value
