import Tkinter
_tk = Tkinter
del Tkinter
from Graphics import *
from Classes import *

def FindUnderline(text):
    import string
    accel = string.find(text, '&')
    while accel != -1:
        text = text[:accel] + text[accel+1:]
        if text[accel] == '&':
            accel = string.find(text, '&', accel+1)
            continue
        break
    return (text, accel)

class TLabel(_tk.Label):
    def __init__(self, owner, **options):
        self.owner = owner
        self.top = options.get('top', 0)
        self.left = options.get('left', 0)
        self.caption = _tk.StringVar()
        (caption, underline) = FindUnderline(options.get('caption', ''))
        self.caption.set(caption)
        self.font = options.get('font', owner.font)
        _tk.Label.__init__(self, owner,
                           textvariable=self.caption,
                           font=self.font,
                           underline=underline,
                           )
        self.place(x=self.left, y=self.top)

class TEdit(_tk.Entry):
    def __init__(self, owner, **options):
        self.top = options.get('top', 0)
        self.left = options.get('left', 0)
        self.width = options.get('width', 160)
        self.height = options.get('height', 24)
        self._textvar = _tk.StringVar()
        self.font = options.get('font', owner.font)
        _tk.Entry.__init__(self, owner,
                           background=options.get('color', clWindow),
                           relief=_tk.SUNKEN,
                           show=options.get('passwordChar', ''),
                           textvariable=self._textvar,
                           font=self.font)
        self.place(x=self.left, y=self.top,
                   width=self.width, height=self.height)


    def __getattr__(self, name):
        if name == 'text':
            return self._textvar.get()
        return getattr(TEdit, name)

    def __setattr__(self, name, value):
        if name == 'text':
            self._textvar.set(value)
        else:
            self.__dict__[name] = value


class TButton(_tk.Button):
    def __init__(self, owner, **options):
        from Forms import TForm
        self.owner = owner
        self._form = owner
        while not isinstance(self._form, TForm):
            self._form = self._form.owner
        self.top = options.get('top', 0)
        self.left = options.get('left', 0)
        self.width = options.get('width', 75)
        self.height = options.get('height', 25)
        self._textvar = _tk.StringVar()
        (caption, underline) = FindUnderline(options.get('caption', ''))
        self.caption = caption
        self.font = options.get('font', owner.font)
        self.onClick = options.get('onClick')
        self.modalResult = options.get('modalResult', '')
        _tk.Button.__init__(self, owner,
                            textvariable=self._textvar,
                            underline=underline,
                            font=self.font,
                            command=self.doClick,
                            )
        self.default = options.get('default', 0)
        self.place(x=self.left, y=self.top, width=self.width)

    def __setattr__(self, name, value):
        if name == 'default':
            self.config(default=(value and 'active' or 'normal'))
        elif name == 'caption':
            self._textvar.set(value)
        self.__dict__[name] = value

    def doClick(self):
        self.onClick and self.onClick(self._form)
        if self.modalResult:
            self._form.modalResult = self.modalResult
        return



class TCheckbutton(_tk.Checkbutton):
    def __init__(self, owner, **options):
        self.top = options.get('top', 0)
        self.left = options.get('left', 0)
        self.width = options.get('width', 160)
        self.height = options.get('height', 24)
        self._intvar = _tk.IntVar()
        self.font = options.get('font', owner.font)
        _tk.Checkbutton.__init__(self, owner,
                                 variable=self._intvar,
                                 font=self.font,
                                 text = options.get('text',''))
        self.place(x=self.left, y=self.top,
                   width=self.width, height=self.height)


    def __getattr__(self, name):
        if name == 'checked':
            return self._intvar.get()
        return getattr(TEdit, name)

    def __setattr__(self, name, value):
        if name == 'checked':
            self._intvar.set(value)
        else:
            self.__dict__[name] = value


class TListBox:
    onClick = None
    onDblClick = None

    def __init__(self, owner, **options):
        self.owner = owner
        self.items = TStringList()
        self.items.onChange = self._itemsUpdate
        self.left = options.get('left', 8)
        self.top = options.get('top', 8)
        self.width = options.get('width', 250)
        self.height = options.get('height', 150)
        self.popupMenu = options.get('popupMenu')
        self.__dict__['selected'] = -1
        
        self._frame = _tk.Frame(owner)
        self._scrollbar = _tk.Scrollbar(self._frame, orient='vertical')
        self._listbox = _tk.Listbox(self._frame,
                                    background=clWindow,
                                    yscrollcommand=self._scrollbar.set,
                                    exportselection='no',
                                    font=owner.font)
        self._scrollbar.configure(command=self._listbox.yview)
        self._scrollbar.pack(side='right', fill='y')
        self._listbox.pack(side='left', fill='both', expand=1)

        self._listbox.bind('<Button-1>', self.doClick)
        self._listbox.bind('<Double-Button-1>', self.doDblClick)
        self._listbox.bind('<Button-3>', self.doMouseDown)
        self._frame.place(x=self.left, y=self.top,
                          width=self.width, height=self.height)

    def __setattr__(self, name, value):
        if name == 'selected':
            self.doSelect(value)
        self.__dict__[name] = value
        return

    def __getattr__(self, name):
        if name == 'selectedString':
            if self.selected == -1: return None
            return self.items.strings[self.selected]
        raise AttributeError(name)


    def _itemsUpdate(self, sender):
        self.redraw()

    def redraw(self):
        # Save the current position in the display
        (top, bottom) = self._listbox.yview()

        self._listbox.delete(0, 'end')
        elements = tuple(self.items.strings)
        self._listbox.tk.call((self._listbox._w, 'insert', 'end') + elements)

        # Restore the display position
        self._scrollbar.set(top, bottom)
        self._listbox.yview('moveto', top)

        self.doSelect(self.selected)

        # Force an update of the lists
        self._frame.update()
        return

    def doSelect(self, index):
        self._listbox.selection_clear(0, self.items.count-1)
        index >= 0 and self._listbox.selection_set(index)
        return

    def doClick(self, event):
        # Primary mouse button
        self.selected = int(event.widget.nearest(event.y))
        self.onClick and self.onClick(self)
        return

    def doDblClick(self, event):
        # Primary mouse button
        self.selected = int(event.widget.nearest(event.y))
        self.onDblClick and self.onDblClick(self)
        return

    def doMouseDown(self, event):
        self.doClick(event)
        if self.popupMenu:
            self.popupMenu.tk_popup(event.x_root, event.y_root)
        return
