#!/usr/bin/env python
########################################################################
#
# File Name:            CorbaUtil.py
#
# Documentation:        http://docs.4suite.org/lib/CorbaUtil.py
#
"""
The CORBA helper functions
WWW: http://4suite.org/lib         e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought, Inc., USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
import FtServer
import CORBA, PortableServer, CosNaming
import os, string
from omniORB import CORBA
import omniORB
import omniORB.PortableServer
from Ft.Server.Server import FtServerServerException, Error
import FtServerFactory__POA, FtServerFactory
import types

orb = None
poa = None
poaManager = None


#NOTE: omniORB 3.0.2 has a bug where the POAManager sometimes prints a useless exception in its __del__
#Just rather annoying
class BugFixPoaManager(omniORB.PortableServer.POAManager):
    base = omniORB.PortableServer.POAManager
    def __del__(self):
        try:
            self.base.__del__(self)
        except:
            pass

omniORB.PortableServer.POAManager = BugFixPoaManager

def Init():
    global orb,poa,poaManger
    if orb:
        return
    #orb = CORBA.ORB_init(['','-ORBtraceLevel','10'], CORBA.ORB_ID)
    orb = CORBA.ORB_init([], CORBA.ORB_ID)
    poa = orb.resolve_initial_references("RootPOA")
    poaManager = poa._get_the_POAManager()
    poaManager.activate()
    return


def GetFactory():

    Init()
    
    try:
        ns = orb.resolve_initial_references("NameService")
    except CORBA.COMM_FAILURE:
        raise FtServerServerException(Error.NO_SERVER_MANAGER)

    nc = CosNaming.NameComponent('FtServerFactory', '')
    fac = ns.resolve([nc])
    return fac._narrow(FtServerFactory.FtServerFactoryIf)



def ActivateObject(imp):
    key = poa.activate_object(imp)
    rt = imp._this()
    #rt._remove_ref()
    return rt
    
def DeactivateObject(imp):
    key = poa.servant_to_id(imp)
    poa.deactivate_object(key)





g_files=[('UserServer_idl','UserServer'),
         ('RdfServer_idl','RdfServer'),
         ('XmlServer_idl','XmlServer'),
         ('MetaUserServer_idl','MetaUserServer'),
         ('MetaXmlServer_idl','MetaXmlServer'),
         ('ServerManager_idl','ServerManager'),
         ]

def OverrideException(klass):
    d_ExceptionImp = list(omniORB.findType(klass._NP_RepositoryId))
    d_ExceptionImp = tuple([d_ExceptionImp[0],klass] + d_ExceptionImp[2:])

    tc_ExceptionImp = omniORB.findTypeCode(klass._NP_RepositoryId)
    omniORB.registerType(klass._NP_RepositoryId, d_ExceptionImp, tc_ExceptionImp)


    for fileName,moduleName in g_files:
        module = __import__(fileName)
        subModuleName = '_0_%s' % moduleName
        subModule = getattr(module,subModuleName)
        for name,value in subModule.__dict__.items():
            if type(value) == types.ClassType and hasattr(value,'__methods__'):
                if value.__name__[:8] == '_objref_':
                    refName = value.__name__[8:]
                    testObject = getattr(subModule,refName)
                else:
                    continue
                for m in value.__methods__:
                    testName =  '_d_'+m
                    if hasattr(testObject,testName):
                        func = getattr(testObject,testName)
                        if func[2] and func[2].has_key(klass._NP_RepositoryId):
                            func[2][klass._NP_RepositoryId] = d_ExceptionImp


