
from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Core import Constants
import FtServerResources__POA, FtServerResources
from Ft.Server.Protocols.CORBA.FtOmniorb.Lib import CorbaUtil, ExceptionUtil
import FtssTypes

class AclObjectImp(FtServerResources__POA.AclObjectIf):


    def __init__(self,res,repo):
        self.resource = res
        repo._registerResource(self)
        self._repo = repo


    def addAcl(self,aclKey,aclIdent):
        return self._runSafe(self.resource.addAcl,(aclKey,aclIdent))

    def getAcl(self,aclKey):
        return self._runSafe(self.resource.getAcl,(aclKey,))

    def getAclIdentifiers(self):
        return self._runSafe(self.resource.getAclIdentifiers,())

    def getAclIdent(self):
        return self._runSafe(self.resource.getAclIdent,())


    def removeAcl(self,aclKey,aclIdent):
        return self._runSafe(self.resource.removeAcl,(aclKey,aclIdent))

    def setAcl(self,aclKey,aclIdent):
        return self._runSafe(self.resource.setAcl,(aclKey,aclIdent))

    def verifyAcl(self,aclKey,testAclIdent):
        return self._runSafe(self.resource.verifyAcl,(aclKey,testAclIdent))

    ###############################
    #ACL Helpers to do common tasks
    ###############################
    def addReadAcl(self,aclIdent):
        return self._runSafe(self.resource.addReadAcl,(aclIdent,))

    def addWorldRead(self):
        return self._runSafe(self.resource.addWorldRead,())

    def addWorldWrite(self):
        return self._runSafe(self.resource.addWorldWrite,())


    def addWriteAcl(self,aclIdent):
        return self._runSafe(self.resource.addWriteAcl,(aclIdent,))

    def getReadAcl(self):
        return self._runSafe(self.resource.getReadAcl,())

    def getWriteAcl(self):
        return self._runSafe(self.resource.getWriteAcl,())

    def removeReadAcl(self,aclIdent):
        return self._runSafe(self.resource.removeReadAcl,(aclIdent,))

    def removeWorldRead(self):
        return self._runSafe(self.resource.removeWorldRead,())

    def removeWorldWrite(self):
        return self._runSafe(self.resource.removeWorldWrite,())

    def removeWriteAcl(self,aclIdent):
        return self._runSafe(self.resource.removeWriteAcl,(aclIdent,))

    def setReadAcl(self,aclIdent):
        return self._runSafe(self.resource.setReadAcl,(aclIdent,))

    def setWorldRead(self):
        return self._runSafe(self.resource.setWorldRead,())

    def setWriteAcl(self,aclIdent):
        return self._runSafe(self.resource.setWriteAcl,(aclIdent,))

    def setWorldWrite(self):
        return self._runSafe(self.resource.setWorldWrite,())



    def getSystemUserContainer(self):
        return self._runSafe(self.resource.getSystemUserContainer,(),self._mapResource)

    def getSystemSuperUserContainer(self):
        return self._runSafe(self.resource.getSystemSuperUserContainer,(),self._mapResource)

    def getSystemGroupContainer(self):
        return self._runSafe(self.resource.getSystemGroupContainer,(),self._mapResource)

    def getSystemDocDefContainer(self):
        return self._runSafe(self.resource.getSystemDocDefContainer,(),self._mapResource)

    def getSystemUserUri(self):
        return self._runSafe(self.resource.getSystemUserUri,())

    def getSystemSuperUserUri(self):
        return self._runSafe(self.resource.getSystemSuperUserUri,())

    def getSystemGroupUri(self):
        return self._runSafe(self.resource.getSystemGroupUri,())

    def getSystemDocDefUri(self):
        return self._runSafe(self.resource.getSystemDocDefUri,())

            
    def _runSafe(self,func,args,returnMap = None):
        inTx = 1
        if not self.resource._xa._db._4ods_inTransaction():
            self.resource._xa._tx.join()
            inTx = 0
        try:
            try:
                rt = apply(func,args)
                if returnMap:
                    rt = returnMap(rt)
                return rt
            except FtServerServerException, e:
                if e.code == Error.OBJECT_DELETED:
                    raise ExceptionUtil.LogObjectHasBeenDeletedException(self.resource._uri)
                raise ExceptionUtil.LogUnknownPythonException()
            except:
                raise ExceptionUtil.LogUnknownPythonException()
        finally:
            if not inTx:
                self.resource._xa._tx.leave()
        


