#include "domlette.h"

#define CHILDNODES_CHECK(df) if (!PyDocumentFragment_Check((df)) || !((PyDocumentFragmentObject *)(df))->childNodes) {PyErr_SetString(g_errorObject, "Document Fragment in inconsistent state"); return NULL;}
#define DOCFRAG_CHECK(doc) CHILDNODES_CHECK((doc));

PyObject *g_documentFragmentNodeType; // added by Brendan Larson 9/12/2002 for Mac OS X 10.1.x compiler compatiblity
PyObject *g_documentFragmentNodeName; // added by Brendan Larson 9/12/2002 for Mac OS X 10.1.x compiler compatiblity


/*Internal Methods*/

PyDocumentFragmentObject *DocumentFragment_CloneNode(PyDocumentFragmentObject *node,int deep,PyObject *newOwnerDocument) {

  long tempId;
  int ctr;
  PyDocumentFragmentObject *newNode;
  PyNodeObject *newChild,*curChild;
  PyObject *childNodes;
  if (!PyDocument_Check(newOwnerDocument)) {
    PyErr_SetString(PyExc_TypeError,"Argument must be a document");
    return NULL;
  }
  newNode = (PyDocumentFragmentObject *)Document_CreateDocumentFragment((PyDocumentObject *)newOwnerDocument,&tempId);
  if (deep) {
    childNodes = PyObject_GetAttrString((PyObject *)node,"childNodes");
    if (!childNodes) return NULL;
    for (ctr=0;ctr < PyList_GET_SIZE(childNodes);ctr ++) {
      curChild = (PyNodeObject *)PyList_GET_ITEM(childNodes,ctr);
      newChild = Node_CloneNode((PyNodeObject *)curChild,deep,newOwnerDocument);
      if (!newChild) return NULL;
      Node_AppendChild((PyNodeObject *)newNode,(PyNodeObject *)newChild);
    }
  }
  return newNode;
}


/*External Methods*/


static struct PyMethodDef DocumentFragment_methods[] = { NODE_METHODS,
  {NULL,     NULL}      /* sentinel */
};



/*Type Methods*/

static PyObject *docfrag_getattr(PyDocumentFragmentObject *self, char *name)
{
  PyObject *rt = NULL;
  int temp;

  DOCFRAG_CHECK(self);

  if (!strcmp(name,"nodeType")) {
    rt = g_documentFragmentNodeType;
  }
  else if (!strcmp(name,"nodeName")) {
    rt = g_documentFragmentNodeName;
  }
  else if (!strcmp(name,"childNodes")) {
    rt = self->childNodes;
  }
  else if (!strcmp(name,"lastChild")) {
    temp = PyList_GET_SIZE(self->childNodes);
    if (temp) {
      rt = PyList_GET_ITEM(self->childNodes,temp-1);
    } else {
      rt = Py_None;
    }
  }
  else if (!strcmp(name,"firstChild")) {
    if (PyList_GET_SIZE(self->childNodes)) {
      rt = PyList_GET_ITEM(self->childNodes,0);
    } else {
      rt = Py_None;
    }
  }
  if (rt) {
    Py_INCREF(rt);
    return rt;
  }
  return node_getattr((PyNodeObject*)self,name,DocumentFragment_methods);
}

static int docfrag_setattr(PyDocumentFragmentObject *self, char *name, PyObject *v)
{
  /* Set attribute 'name' to value 'v'. v==NULL means delete */
  if (v == NULL) {
    PyErr_Format(PyExc_AttributeError, 
                 "Cannot delete attribute '%.400s' on '%.50s' object",
                 name, self->ob_type->tp_name);
  } else {
    PyErr_Format(PyExc_AttributeError,
                 "Cannot set attribute '%.400s' on '%.50s' object", 
                 name, self->ob_type->tp_name);
  }
  return -1;
}

static void docfrag_dealloc(PyDocumentFragmentObject *node)
{
  PyObject_GC_UnTrack((PyObject *) node);

  Py_XDECREF(node->childNodes);
  node->childNodes = NULL;

  node_dealloc((PyNodeObject*)node);
}

static int docfrag_clear(PyDocumentFragmentObject *self)
{
  /*Called when we need to break cycles*/
  Py_XDECREF(self->childNodes);
  self->childNodes = NULL;
  return node_clear((PyNodeObject *)self);
}

static int docfrag_traverse(PyDocumentFragmentObject *self, visitproc visit, void *arg)
{
  int rt;
  if (self->childNodes != NULL) {
    rt = visit(self->childNodes, arg);
    if (rt) return rt;
  }
  return node_traverse((PyNodeObject *)self,visit,arg);
}



static PyObject *docfrag_repr(PyDocumentFragmentObject *docfrag)
{
  char buf[256];
  sprintf(buf, "<cDocumentFragment at %p>", docfrag);
  return PyString_FromString(buf);
}

static int docfrag_print(PyDocumentFragmentObject *v, FILE *fp, int flags)
/* flags not used but required by interface */
{
  PyObject *repr = docfrag_repr(v);
  fprintf(fp, PyString_AS_STRING(repr));
  Py_DECREF(repr);
  return 0;
}


PyTypeObject PyDomletteDocumentFragment_Type = {
    PyObject_HEAD_INIT(0)
    0,
    "cDomlette.DocumentFragment",
    GC_TP_BASICSIZE(PyDocumentFragmentObject),
    0,
    (destructor)docfrag_dealloc,    /*tp_dealloc*/
    (printfunc)docfrag_print,   /*tp_print*/
    (getattrfunc)docfrag_getattr,   /*tp_getattr*/
    (setattrfunc)docfrag_setattr,   /*tp_setattr*/
    0,                          /*tp_compare*/
    (reprfunc)docfrag_repr,     /*tp_repr*/
    0,                          /*tp_as_number*/
    0,              /*tp_as_sequence*/
    0,              /*tp_as_mapping*/
    0,                             /*tp_hash*/
    0,          /*tp_call*/
    0,          /*tp_str*/
    0,                      /*tp_getattro*/
    0,          /*tp_setattro*/
    0,                          /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,                          /* tp_doc */
    (traverseproc)docfrag_traverse,  /* tp_traverse */
    (inquiry)docfrag_clear,          /* tp_clear */
    0,                          /* tp_richcompare */
    0,                          /* tp_weaklistoffset */
};

