#include "domlette.h"

/*Internal Interfaces*/


/*External Interfaces*/

static char PyDOMImplementation_hasFeature__doc__[] =
"See if this implementation supports a given feature";
PyObject *PyDOMImplementation_hasFeature(PyObject * self, PyObject * args) {
/*     _features = {'CORE' : 2.0, */
/*                  'XML' : 2.0, */
/*                  } */

/*     def hasFeature(self, feature, version=''): */
/*         fv = self._features.get(string.upper(feature)) */
/*         if fv: */
/*             if version: */
/*                 try: */
/*                     return float(version) == fv */
/*                 except: */
/*                     return 0 */
/*             return 1 */
/*         return 0 */


  char *feature;
  char *version;
  PyObject *res;

  if(!PyArg_ParseTuple(args,"ss:hasFeature",&feature,&version))
    return NULL;

  if (strcasecmp(feature,"core") ) {
    res = PyInt_FromLong(0);
  }
  else if (atoi(version) <= 2.0 ) {
    res = PyInt_FromLong(1);
  } else {
    res = PyInt_FromLong(0);
  }
  return res;
}


static char PyDOMImplementation_createDocumentType__doc__[] =
"Create a new document type instance";
PyObject *PyDOMImplementation_createDocumentType(PyObject * self, PyObject * args)
{
  register PyDocTypeObject *doctype;
  PyObject *arg0, *arg1, *arg2;
  PyObject *qualifiedName, *publicId, *systemId;

  doctype = PyObject_GC_New(PyDocTypeObject, &PyDomletteDocType_Type);

  if(!PyArg_ParseTuple(args, "OOO:createDocumentType", &arg0, &arg1, &arg2))
    return NULL;

  qualifiedName = PyUnicode_FromObject(arg0);
  if (!qualifiedName) {
    if (PyErr_ExceptionMatches(PyExc_TypeError))
      PyErr_Format(PyExc_TypeError, 
                   "qualifiedName must be unicode or string, %.80s found.",
                   arg0->ob_type->tp_name);
    return NULL;
  }

  if (arg1 != Py_None) {
    publicId = PyUnicode_FromObject(arg1);
    if (!publicId) {
      if (PyErr_ExceptionMatches(PyExc_TypeError))
        PyErr_Format(PyExc_TypeError, 
                     "publicId must be None, unicode or string, %.80s found.",
                     arg1->ob_type->tp_name);
      return NULL;
    }
  } else {
    publicId = arg1;
    Py_INCREF(publicId);
  }

  if (arg2 != Py_None) {
    systemId = PyUnicode_FromObject(arg2);
    if (!systemId) {
      if (PyErr_ExceptionMatches(PyExc_TypeError))
        PyErr_Format(PyExc_TypeError, 
                     "systemId must be None, unicode or string, %.80s found.",
                     arg2->ob_type->tp_name);
      return NULL;
    }
  } else {
    systemId = arg2;
    Py_INCREF(systemId);
  }

  Node_INIT((PyNodeObject*)doctype, Py_None, &PyDomletteDocType_Type);

  doctype->publicId = publicId;
  doctype->systemId = systemId;
  doctype->nodeName = qualifiedName;
  doctype->internalSubset = Py_None;
  Py_INCREF(Py_None);

  PyObject_GC_Track(doctype);

  return (PyObject *)doctype;
}


static char PyDOMImplementation_createDocument__doc__[] =
"Create a new document instance";
PyObject *PyDOMImplementation_createDocument(PyObject * self, PyObject * args)
{
  PyObject *arg0, *arg1, *doctype;
  PyObject *namespaceURI, *qualifiedName;
  PyDocumentObject *doc;
  unsigned long index;

  if(!PyArg_ParseTuple(args,"OOO:createDocument", &arg0, &arg1, &doctype))
    return NULL;

  /* validate the arguments */
  if (arg0 != Py_None) {
    namespaceURI = PyUnicode_FromObject(arg0);
    if (!namespaceURI) {
      if (PyErr_ExceptionMatches(PyExc_TypeError))
        PyErr_Format(PyExc_TypeError, 
                     "namespaceURI must be None, unicode or string, %.80s found.",
                     arg0->ob_type->tp_name);
      return NULL;
    }
  } else {
    namespaceURI = arg0;
    Py_INCREF(Py_None);
  }

  if (arg1 != Py_None) {
    qualifiedName = PyUnicode_FromObject(arg1);
    if (!qualifiedName) {
      if (PyErr_ExceptionMatches(PyExc_TypeError))
        PyErr_Format(PyExc_TypeError, 
                     "qualifiedName must be None, unicode or string, %.80s found.",
                     arg1->ob_type->tp_name);
      return NULL;
    }
  } else {
    qualifiedName = arg1;
    Py_INCREF(Py_None);
  }

  if (doctype != Py_None && !PyDocType_Check(doctype)) {
      PyErr_Format(PyExc_TypeError, 
                   "doctype must be None or a DocumentType object , %.80s found.",
                   doctype->ob_type->tp_name);
      return NULL;
  }
  Py_INCREF(doctype);

  doc = Document_NEW(&index, NULL);

  Py_DECREF(doc->docType);
  doc->docType = doctype;

  /* Add the doctype as a child if not None */
  if (doctype != Py_None) {
    Node_AppendChild((PyNodeObject *)doc, (PyNodeObject *)doctype);
  }

  /* See if we need to add a documentElement */
  if (qualifiedName != Py_None) {
    PyObject *prefix, *localName;
    PyElementObject *elem;

    SplitQName(qualifiedName, &prefix, &localName);
    elem = Document_CreateElementNS(doc, namespaceURI, prefix, localName, &index);

    Py_DECREF(prefix);
    Py_DECREF(localName);

    Node_AppendChild((PyNodeObject *)doc, (PyNodeObject *)elem);
  }

  Py_DECREF(namespaceURI);
  Py_DECREF(qualifiedName);

  return (PyObject *)doc;
}


static struct PyMethodDef DOMImplementation_methods[] = {
  {"hasFeature",   (PyCFunction)PyDOMImplementation_hasFeature,  METH_VARARGS, PyDOMImplementation_hasFeature__doc__},
  {"createDocumentType",   (PyCFunction)PyDOMImplementation_createDocumentType,  METH_VARARGS, PyDOMImplementation_createDocumentType__doc__},
  {"createDocument",   (PyCFunction)PyDOMImplementation_createDocument,  METH_VARARGS, PyDOMImplementation_createDocument__doc__},
  {NULL,     NULL}      /* sentinel */
};


/*
  Type Interfaces
*/

static PyObject *domimp_getattr(PyDOMImplementationObject *self, char *name)
{
  return Py_FindMethod(DOMImplementation_methods, (PyObject *)self, name);
}

static int domimp_setattr(PyDOMImplementationObject *self, char *name, PyObject *v)
{
  /* Set attribute 'name' to value 'v'. v==NULL means delete */
  if (v == NULL) {
    PyErr_Format(PyExc_AttributeError, 
                 "Cannot delete attribute '%.400s' on '%.50s' object",
                 name, self->ob_type->tp_name);
  } else {
    PyErr_Format(PyExc_AttributeError,
                 "Cannot set attribute '%.400s' on '%.50s' object", 
                 name, self->ob_type->tp_name);
  }
  return -1;
}

static void
domimp_dealloc(PyDOMImplementationObject *node)
{
  PyMem_DEL(node);
}


static PyObject *domimp_repr(PyDOMImplementationObject *domimp)
{
  char buf[256];
  sprintf(buf, "<cDOMImplementation at %p>", domimp);
  return PyString_FromString(buf);
}


static int domimp_print(PyDOMImplementationObject *v, FILE *fp, int flags)
{
  PyObject *repr = domimp_repr(v);
  fprintf(fp, PyString_AS_STRING(repr));
  Py_DECREF(repr);
  return 0;
}


PyTypeObject PyDomletteDOMImplementation_Type = {
    PyObject_HEAD_INIT(0)
    0,
    "cDomlette.DOMImplementation",
    sizeof(PyDOMImplementationObject),
    0,
    (destructor)domimp_dealloc,   /*tp_dealloc*/
    (printfunc)domimp_print,  /*tp_print*/
    (getattrfunc)domimp_getattr,  /*tp_getattr*/
    (setattrfunc)domimp_setattr,  /*tp_setattr*/
    0,                          /*tp_compare*/
    (reprfunc)domimp_repr,          /*tp_repr*/
    0,                          /*tp_as_number*/
    0,              /*tp_as_sequence*/
    0,              /*tp_as_mapping*/
    0,                             /*tp_hash*/
    0,          /*tp_call*/
    0,          /*tp_str*/
    0,                      /*tp_getattro*/
    0,          /*tp_setattro*/
};

